// $Header: /project/A3Pro/asic/asic_usb.c,v 1.28 2003/05/07 10:10:00 dan Exp $
// Draft by Dan 2003-3-21
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <dirent.h>
#include <linux/usb.h>

#include "asic_usb.h"


STATUS WriteIOControl(PAsic chip, WORD wValue, WORD wIndex, WORD wLength, LPBYTE lpbuf)
{
	STATUS status = STATUS_GOOD;
	int ret;
	cmsg c;

	c.req.requesttype =0x40;
    c.req.request = 0x01;
    c.req.value = wValue;
    c.req.index = wIndex;
    c.req.length = wLength;
    c.data = lpbuf;


	ret = ioctl( chip->fd,
	       SCANNER_IOCTL_CTRLMSG,
		   &c);

	if (ret<0)
	{
		XDBG(DEBUG_ASIC,"WriteIOControl: ioctl error");
		return STATUS_IO_ERROR;
	}

	return STATUS_GOOD;
}

STATUS ReadIOControl(PAsic chip, WORD wValue, WORD wIndex, WORD wLength, LPBYTE lpbuf)
{
	STATUS status = STATUS_GOOD;

	cmsg c;

	c.req.requesttype =0xc0;
    c.req.request = 0x01;
    c.req.value = wValue;
    c.req.index = wIndex;
    c.req.length = wLength;
    c.data = lpbuf;

	ioctl(chip->fd,SCANNER_IOCTL_CTRLMSG, &c);

	return status;
}

STATUS Mustek_SetDMAReadScanMode(PAsic chip)
{
	STATUS status;
	BYTE buf[1];
	buf[0] = 1;//dummy, do not care

	XDBG(DEBUG_ASIC,"Mustek_SetDMAReadScanMode: Enter");

	status = WriteIOControl(chip,0x01,0,1,buf);

	XDBG(DEBUG_ASIC,"Mustek_SetDMAReadScanMode: Exit");

	return status;
}

STATUS Mustek_SendData(PAsic chip, BYTE reg, BYTE data)
{
	BYTE buf[2];
	STATUS status;
	
//	XDBG(DEBUG_ASIC,"Mustek_SendData: Start\n");

	buf[0] = reg; 
	buf[1] = data;

	status = WriteIOControl(chip,0xb0,0,2,buf);
	
	if (status != STATUS_GOOD)
		perror("Mustek_SendData: write error\n");
	
	//XDBG(DEBUG_ASIC,"Mustek_SendData: Exit");

	return status;
}

STATUS Mustek_ReceiveData(PAsic chip, LPBYTE reg)
{
	STATUS status;
	
	status = ReadIOControl(chip, 0x07, 0, 1, reg);
	
	return status;
}

STATUS Mustek_WriteAddressLineForRegister(PAsic chip, BYTE x)
{
	STATUS status;
	
	status = WriteIOControl(chip,0x04,x,1,&x);
	
	return status;
}

STATUS Mustek_DMAReadGeneralMode(PAsic chip, DWORD size,LPBYTE lpdata)
{
	STATUS status;
	DWORD i,buf[1],rate;
	DWORD write_size;
	
	XDBG(DEBUG_ASIC,"Mustek_DMAReadGeneralMode: Enter");

	buf[0] = write_size = 2*1024*1024;

	//tell the USB interface circuit read size, hardware maximum 64k,(2 bytes counter)
	for(i=0;i<size/(write_size);i++)
	{
		status = WriteIOControl(chip,0x03,0,4,(LPBYTE)(buf));
		if (status != STATUS_GOOD)
			return status;
		
		rate = USB_DMARead(chip,write_size,lpdata + i*write_size);
	}

	buf[0] = size - i*write_size;

	status = WriteIOControl(chip, 0x03,0,4,(LPBYTE)(buf));
	if (status != STATUS_GOOD)
			return status;

	USB_DMARead(chip,size - i*write_size,lpdata + i*write_size);
	
	XDBG(DEBUG_ASIC,"Mustek_DMAReadGeneralMode: Exit");

	return STATUS_GOOD;
}

STATUS Mustek_DMAReadScanMode(PAsic chip, DWORD size, LPBYTE lpdata)
{
	STATUS status = STATUS_GOOD;
	DWORD i;
	DWORD temp_a;

	XDBG(DEBUG_ASIC,"Mustek_DMAReadScanMode: Enter:size=%d",size);
	
	status = USB_DMARead(chip,size,lpdata);
	
	if (status != STATUS_GOOD)
	{
		XDBG(DEBUG_ASIC,"Mustek_DMAReadScanMode: USB_DMARead return Error");
	}

	XDBG(DEBUG_ASIC,"Mustek_DMAReadScanMode: Exit");

	return status;
}

STATUS Mustek_DMAWrite(PAsic chip,DWORD size, LPBYTE lpdata)
{
	STATUS status;
	DWORD buf[1];
	DWORD i;
	DWORD write_size;

	//tell the USB interface circuit write size, hardware maximum 64k,(2 bytes counter)

	buf[0] = size;

	status = WriteIOControl(chip,0x02,0,4,(LPBYTE)(buf)); // Modified by dan
	if (status != STATUS_GOOD)
		return status;

	write_size = 60*1024;
	for(i=0;i<size/(write_size);i++)
	{
		status = USB_DMAWrite(chip,write_size,lpdata + i*write_size);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Mustek_DMAWrite: USB_DMAWrite error");
			return status;
		}

		usleep(20000);
	}

	status = USB_DMAWrite(chip,size - i*write_size,lpdata + i*write_size);
	if (status != STATUS_GOOD)
	{
		XDBG(DEBUG_ASIC,"Mustek_DMAWrite: USB_DMAWrite error");
		return status;
	}

	usleep(20000);

	return STATUS_GOOD;
}

STATUS USB_DMAWrite(PAsic chip,DWORD size, LPBYTE lpdata)
{
	STATUS status ;
//	LPBYTE USB_temp_memory;
	
//	USB_temp_memory = (LPBYTE)malloc(size);

	status = RawWriteCommand(chip,(LPVOID)lpdata,size);
//	USB_io_nBytes = 0;
//	memcpy(USB_temp_memory, lpdata,size);

//	status = RawWriteCommand(chip,USB_temp_memory, size);

//	free(USB_temp_memory);
	
	return status;
}


STATUS USB_DMARead(PAsic chip, DWORD size, LPBYTE lpdata)
{
	STATUS status;
	DWORD i;
	DWORD temp_a;



	XDBG(DEBUG_ASIC,"USB_DMARead: Enter size=%d",size);


	temp_a = TEMP_MEMORY_SIZE_128K;

	for(i=0;i<(size/temp_a);i++)
	{
	    status = RawReadData(chip,lpdata+i*TEMP_MEMORY_SIZE_128K, temp_a);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"USB_DMARead: RawReadData: error");
			return status;
		}
			

	}

	status = RawReadData(chip, lpdata+i*TEMP_MEMORY_SIZE_128K,size - i*TEMP_MEMORY_SIZE_128K);
	if (status != STATUS_GOOD)
	{
		XDBG(DEBUG_ASIC,"USB_DMARead: RawReadData: error");
		return status;
	}



	XDBG(DEBUG_ASIC,"USB_DMARead: Exit");
	
	return STATUS_GOOD;
}

STATUS RawWriteCommand(PAsic chip,LPVOID lpBuffer,DWORD nByteCount)
{
	STATUS status;
	int size;

	XDBG(DEBUG_ASIC,"RawWriteCommand: write %d bytes",nByteCount);
	if (lpBuffer != NULL)
		size = write(chip->fd, lpBuffer, nByteCount);
	else
	{
		XDBG(DEBUG_ASIC,"RawWriteCommand: Buffer is null");
		return STATUS_MEM_ERROR;
	}
	if (size<0)
	{
		XDBG(DEBUG_ASIC,"RawWriteCommand: Write Error");
		return STATUS_IO_ERROR;
	}

	XDBG(DEBUG_ASIC,"RawWriteCommand: Write OK");
	return STATUS_GOOD;
}

STATUS RawReadData(PAsic chip, LPBYTE lpBuffer,DWORD ByteCount)
{
	STATUS status;
	int read_size;
	BYTE* buffer;

	XDBG(DEBUG_ASIC,"RawReadData: read %d bytes",ByteCount);

	
	if (lpBuffer != NULL && ByteCount>=0)
	{
		buffer = (BYTE*)malloc(sizeof(BYTE)*ByteCount+1);
		read_size = read(chip->fd,buffer,ByteCount);
		XDBG(DEBUG_ASIC,"RawReaddata: after read from kernel");
		memcpy(lpBuffer,buffer,ByteCount);
		free(buffer);
	}
	else
	{
		XDBG(DEBUG_ASIC,"RawReadData: Buffer is null or ByteCount is nod position");
		return STATUS_MEM_ERROR;
	}
	if (read_size<0)
	{
		XDBG(DEBUG_ASIC,"RawReadData: Io error");
		return STATUS_IO_ERROR;
	}
	XDBG(DEBUG_ASIC,"RawReadData: Real read %d bytes",read_size);
	return STATUS_GOOD;
}

/*
STATUS DeviceIOControl(PAsic chip, UINT cmd, LPVOID io_in,
					   DWORD BlockInSize, LPVOID io_out,
					   DWORD BlockOutSize,LPDWORD ByteReturn)
{
	STATUS status;
	
	cmsg c;
	
	return STATUS_GOOD;
}
*/





















