/* sane - Scanner Access Now Easy.
	Copyright (C) 2005 Mustek.
	Originally maintained by Mustek
	Author:Jason 2003.04  Roy 2005.09

	This file is part of the SANE package.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA.

	As a special exception, the authors of SANE give permission for
	additional uses of the libraries contained in this release of SANE.

	The exception is that, if you link a SANE library with other files
	to produce an executable, this does not by itself cause the
	resulting executable to be covered by the GNU General Public
	License.  Your use of that executable is in no way restricted on
	account of linking the SANE library code into it.

	This exception does not, however, invalidate any other reasons why
	the executable file might be covered by the GNU General Public
	License.

	If you submit changes to SANE to the maintainers to be included in
	a subsequent release, you agree by submitting the changes that
	those changes may be distributed with this exception intact.

	If you write modifications of your own for SANE, it is your choice
	whether to permit this exception to apply to your modifications.
	If you do not wish that, delete this exception notice.

	This file provides a generic USB interface.  */


#include <pthread.h>
#include <stdlib.h>

#include "../include/typedef.h"
#include "../include/asic.h"
#include "../include/public.h"
#include "../include/spicall.h"
#include "../include/ini.h"
#include "../include/debug.h"
#include "MustScanner.h"
#include "ScannerDefine.h"
#include "spicall_g.h"




extern unsigned short* g_pGammaTable;
extern char* g_pDeviceFile;
extern WORD g_wLineartThreshold;
extern BOOL g_bSharpen;


BOOL MustScanner_Init()
{

	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Init: Asic_Open return error");

		return FALSE;
	}
	if(Asic_Initialize(&g_chip)!=STATUS_GOOD)
	{	
		XDBG(DEBUG_SPICALL,"MustScanner_Init: Asic_Initialize return error");

		return FALSE;
	}

	g_dwImageBufferSize=24L*1024L*1024L;
	g_dwBufferSize = 64L*1024L*2;
	g_lpReadImageHead=NULL;

	g_isCanceled=FALSE;
	g_bFirstReadImage=TRUE;
	g_bOpened=FALSE;
	g_bPrepared=FALSE;
	g_bSharpen=FALSE;

	g_isScanning=FALSE;
	g_isSelfGamma=FALSE;
	g_pGammaTable=NULL;

	if(g_pDeviceFile!=NULL)
	{
		free(g_pDeviceFile);
		g_pDeviceFile=NULL;
	}

	g_ssScanSource = SS_Reflective;
	g_PixelFlavor = PF_BlackIs0;

	g_dwCalibrationSize = 64L*1024L;

	Asic_Close(&g_chip);	
	
	XDBG(DEBUG_SPICALL,"MustScanner_Init: leave MustScanner_Init");

	return TRUE;
}

BOOL MustScanner_GetScannerState()
{

	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_GetScannerState: Asic_Open return error");

		return FALSE;
	}
	else
	{
		Asic_Close(&g_chip);
		return TRUE;
	}	
}

BOOL MustScanner_PowerControl(BOOL isLampOn,BOOL isTALampOn)
{
	BOOL hasTA;

	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_Open return error");

		return FALSE;
	}

	
	if(Asic_TurnLamp(&g_chip,isLampOn)!=STATUS_GOOD)
	{	
		XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_TurnLamp return error");

		return FALSE;
	}

	if(Asic_IsTAConnected(&g_chip,&hasTA)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_IsTAConnected return error");

		return FALSE;
	}

	if(hasTA)
	{
		if(Asic_TurnLampEngineTA(&g_chip,isTALampOn)!=STATUS_GOOD)
		{
			XDBG(DEBUG_SPICALL,"MustScanner_PowerControl: Asic_TurnLampEngineTA return error");

			return FALSE;
		}
	}	

	Asic_Close(&g_chip);
	return TRUE;
}

BOOL MustScanner_BackHome()
{
	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: call in ");

	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
    	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: Asic_Open return error");

		return FALSE;
	}
	if(Asic_CarriageHome(&g_chip,FALSE)!=STATUS_GOOD)
	{
    	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: Asic_CarriageHome return error");

		return FALSE;
	}
	if(Asic_WaitUnitReady(&g_chip)!=STATUS_GOOD)
	{
    	XDBG(DEBUG_SPICALL,"MustScanner_BackHome: Asic_WaitUnitReady return error");
		
		return FALSE;
	}
	Asic_Close(&g_chip);
	return TRUE;

}

BOOL MustScanner_Prepare(BYTE bScanSource)
{	
	XDBG(DEBUG_SPICALL,"MustScanner_Prepare: call in");

	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_Open return error");

		return FALSE;
	}
	if(Asic_WaitUnitReady(&g_chip)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_WaitUnitReady return error");

		return FALSE;
	}
	if(bScanSource==SS_Reflective)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare:ScanSource is SS_Reflective");
		if(Asic_TurnLamp(&g_chip,TRUE)!=STATUS_GOOD)
		{	
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_TurnLamp return error");
			return FALSE;
		}

		if(Asic_SetSource(&g_chip,LS_REFLECTIVE)!=STATUS_GOOD)
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return error");
			return FALSE;
		}

	}
	else if(bScanSource==SS_Positive)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare:ScanSource is SS_Positive");

		if(Asic_TurnTA(&g_chip,TRUE)!=STATUS_GOOD)
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_TurnTA return error");
			return FALSE;
		}
		if(Asic_SetSource(&g_chip,LS_POSITIVE)!=STATUS_GOOD)
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return error");
			return FALSE;
		}
	}
	else if(bScanSource==SS_Negative)
	{	
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare:ScanSource is SS_Negative");

		if(Asic_TurnTA(&g_chip,TRUE)!=STATUS_GOOD)
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_TurnTA return error");
			return FALSE;
		}

		if(Asic_SetSource(&g_chip,LS_NEGATIVE)!=STATUS_GOOD)
		{
			XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return error");
			return FALSE;
		}
		XDBG(DEBUG_SPICALL,"MustScanner_Prepare: Asic_SetSource return good");
		
	}

	Asic_Close(&g_chip);

	g_bPrepared = TRUE;

	return TRUE;
}

BOOL MustScanner_AdjustOffset(int nTimes,BOOL *bDirection,BYTE *bOffset,
							  BYTE *bLastMin,BYTE *bLastOffset,WORD *wMinValue, 
							  BYTE *bOffsetUpperBound, BYTE *bOffsetLowerBound, 
							  WORD wStdMinLevel,WORD wStdMaxLevel)
{

	
	if((*wMinValue <= wStdMaxLevel) && (*wMinValue >= wStdMinLevel))
	{
		return TRUE;
	}
	
	if(nTimes == 0)
	{
		*bLastMin = LOBYTE(*wMinValue);
		*bLastOffset = *bOffset;
		
		if(*wMinValue == 255)  
		{
			*bOffset = 0; 
		}
		else
		{
			*bOffset = 255; 
		}
	}

	if(nTimes == 1)
	{
		if(*wMinValue > *bLastMin)
		{
			if(*wMinValue > wStdMaxLevel && *bLastMin > wStdMaxLevel)
			{
				*bDirection = !(*bDirection); 
				return TRUE;
			}
			
			if(*wMinValue < wStdMinLevel && *bLastMin < wStdMinLevel)
				return TRUE;
		}
							
		if(*wMinValue < *bLastMin)
		{
			if(*wMinValue < wStdMinLevel && *bLastMin < wStdMinLevel)
				*bDirection = !(*bDirection); 
			
			if(*wMinValue > wStdMaxLevel && *bLastMin > wStdMaxLevel)
				return TRUE;
		}
	}
	
	if(nTimes > 1)
	{
		if(*wMinValue > *bLastMin)
		{
			BYTE bTemp;
			
			bTemp = *bOffset;
			
			*bOffset =  (*bOffsetUpperBound + *bOffsetLowerBound) / 2;
			
			if(nTimes > 2)
			{
				if(*wMinValue > wStdMaxLevel)
					if(bDirection)
						*bOffsetLowerBound = bTemp;
					else
						*bOffsetUpperBound = bTemp;
					else
						if(bDirection)
							*bOffsetUpperBound = bTemp;
						else
							*bOffsetLowerBound = bTemp;
			}
			
			*bLastOffset = bTemp;
			*bLastMin = (BYTE) *wMinValue;
			
		}
		else
		{
			BYTE bTemp;
			
			bTemp = *bOffset;
			
			*bOffset =  (*bOffsetUpperBound + *bOffsetLowerBound) / 2;
			
			if(nTimes > 2)
			{
				if(*wMinValue == *bLastMin)
				{
					if(*wMinValue > wStdMaxLevel)
					{
						if(!bDirection)
							*bOffsetUpperBound = bTemp;
						else
							*bOffsetLowerBound = bTemp;
					}
					else
					{
						if(!bDirection)
							*bOffsetLowerBound = bTemp;
						else
							*bOffsetUpperBound = bTemp;
					}
				}
				else
				{
					if(*wMinValue > wStdMaxLevel)
					{
						if(bDirection)
							*bOffsetUpperBound = bTemp;
						else
							*bOffsetLowerBound = bTemp;
					}
					else
					{
						if(bDirection)
							*bOffsetLowerBound = bTemp;
						else
							*bOffsetUpperBound = bTemp;
					}
				}
			}
			
			*bLastOffset = bTemp;
			*bLastMin = (BYTE) *wMinValue;
			
		}
	} 

	return TRUE;
}

BOOL MustScanner_SecondAdjustOffset(int nTimes, BOOL *bDirection, BYTE *bOffset, 
									BYTE *bLastMin, BYTE *bLastOffset, WORD *wMinValue, 
									BYTE *bOffsetUpperBound, BYTE *bOffsetLowerBound, 
									WORD wStdMinLevel,WORD wStdMaxLevel)
{
	
	if((*wMinValue <= wStdMaxLevel) && (*wMinValue >= wStdMinLevel))
	{
		return TRUE;
	}
	if(nTimes == 0)
	{
		*bLastMin = LOBYTE(*wMinValue);
		*bLastOffset = *bOffset;
		
		if(*bDirection == 0)
		{
			*bOffsetUpperBound = *bLastOffset;
			*bOffsetLowerBound = 0;
			*bOffset = 0; 
		}
		else
		{
			*bOffsetUpperBound = 255;
			*bOffsetLowerBound = *bLastOffset;
			*bOffset = 255; 
		}
	}
	
	if(nTimes >= 1)
	{
		if(*wMinValue > wStdMaxLevel)
		{
			if(*wMinValue > *bLastMin)
			{
				if(*bDirection == 0)
				{
					*bOffsetUpperBound = *bOffset;
				}
				else
				{
					*bOffsetLowerBound = *bOffset;
				}
				
				*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
			}
			else
			{
				if(*bDirection == 1)
				{
					*bOffsetUpperBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
				else
				{
					*bOffsetLowerBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
			}
		}
		
		
		if(*wMinValue < wStdMinLevel)
		{
			if(*wMinValue > *bLastMin)
			{
				if(*bDirection == 0)
				{
					*bOffsetLowerBound = *bOffset;
				}
				else
				{
					*bOffsetUpperBound = *bOffset;
				}
				
				*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
			}
			else
			{
				if(*bDirection == 1)
				{
					*bOffsetUpperBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
				else
				{
					*bOffsetLowerBound = *bOffset;
					*bOffset = (*bOffsetUpperBound + *bOffsetLowerBound) / 2; 
				}
			}
		} 
		*bLastMin = (BYTE) *wMinValue;
	} 
}

WORD MustScanner_FiltLower(WORD* pSort,WORD TotalCount,WORD LowCount,WORD HighCount)
{
		
	WORD Bound=TotalCount-1;
	WORD LeftCount=HighCount-LowCount;
	int Temp=0;
	DWORD Sum=0;
	WORD i,j;

	for (i=0;i<Bound;i++)
	{
		for (j=0;j<Bound-i;j++) 
		{
			if (pSort[j+1]>pSort[j]) 
			{
				Temp=pSort[j];
				pSort[j]=pSort[j+1];
				pSort[j+1]=Temp;
			}
		}
	}

	for (i=0 ; i<LeftCount ; i++)
		Sum+=pSort[i+LowCount];
	return (WORD)(Sum/LeftCount);

}

BOOL MustScanner_GetRgb48BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{

	WORD wWantedTotalLines;
	WORD TotalXferLines;
	WORD wRLinePos=0;
	WORD wGLinePos=0;
	WORD wBLinePos=0;
	WORD wRTempData;
	WORD wGTempData;
	WORD wBTempData;
	WORD i;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	TotalXferLines=0;	


	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: call in ");

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}



	if(!isOrderInvert)
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_Width; i++)
				{
					wRTempData =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+0); 
					wRTempData+=*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+1)<<8; 
					wGTempData =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+2); 
					wGTempData+=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+3)<<8; 
					wBTempData =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+4); 
					wBTempData+=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+5)<<8;
					*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wBTempData+131072]);
					*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wBTempData+131072]);
				}					
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;
				
			}

			if(g_isCanceled)
			{

				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");

				break;
			}
		}
	}
	
	else		
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					wRTempData =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+0);
					wRTempData+=*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*6+1)<<8;
					wGTempData =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+2);
					wGTempData+=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*6+3)<<8; 
					wBTempData =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+4); 
					wBTempData+=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*6+5)<<8; 
					*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wRTempData]);
					*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
					*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wBTempData+131072]);
					*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wBTempData+131072]);
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;				
				
			}
			if(g_isCanceled)
			{	

				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine: thread exit");
				break;
			}
		}
	}	

	*wLinesCount = TotalXferLines;

	g_isScanning=FALSE;
	return TRUE;
}

BOOL MustScanner_GetRgb48BitLine1800DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	
	WORD wRLinePosOdd=0;
	WORD wGLinePosOdd=0;
	WORD wBLinePosOdd=0;
	WORD wRLinePosEven=0;
	WORD wGLinePosEven=0;
	WORD wBLinePosEven=0;	
	WORD wRTempData;
	WORD wGTempData;
	WORD wBTempData;
	WORD i;
	BOOL isSoftwareCalibration;	

	TotalXferLines=0;
	wWantedTotalLines=*wLinesCount;

	g_isCanceled=FALSE;
	g_isScanning=TRUE;

	

	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1800DPI: call in ");

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1800DPI: thread create");

		g_bFirstReadImage=FALSE;
	}

#ifdef HARDWARE_SHADING_SWITCH_ON
	if(g_Width > HARDWARE_SHADING_WIDTH * 1800)
		isSoftwareCalibration = TRUE;
	else 
		isSoftwareCalibration = FALSE;
#else
	isSoftwareCalibration = TRUE;
#endif

	if(!isOrderInvert)
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1800DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				
				if(isSoftwareCalibration)
				{
					WORD wDoShadingLine;
					wDoShadingLine = (WORD)g_dwScannedTotalLines - g_wReadyShadingLine;
					
					if(wDoShadingLine)  
					{
						
						if((g_wStartShadingLinePos + wDoShadingLine) >= g_wMaxScanLines)  
						{
							
							WORD wFirstShadingLines;
							
							wFirstShadingLines = g_wMaxScanLines - g_wStartShadingLinePos;
							
							wDoShadingLine = wDoShadingLine - wFirstShadingLines;
							
							Asic_DoSoftwareShadingRGB48Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wFirstShadingLines);
							
							g_wReadyShadingLine += wFirstShadingLines;
							g_wStartShadingLinePos = 0;
							
							Asic_DoSoftwareShadingRGB48Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						}
						else
						{
							Asic_DoSoftwareShadingRGB48Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						}
						
						g_wReadyShadingLine += wDoShadingLine;
						g_wStartShadingLinePos += wDoShadingLine;
					}
					
				}
				
				
				wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				
				for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						wRTempData =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+0); 
						wRTempData+=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+1)<<8; 
						wGTempData =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+2); 
						wGTempData+=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+3)<<8; 
						wBTempData =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+4);
						wBTempData+=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+5)<<8; 
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wBTempData+131072]);
						i++;
						wRTempData =*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+0); 
						wRTempData+=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+1)<<8; 
						wGTempData =*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+2); 
						wGTempData+=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+3)<<8; 
						wBTempData =*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+4);
						wBTempData+=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+5)<<8;
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wBTempData+131072]);
						i++;
						
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;
				
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1800DPI: thread exit");
				break;
			}
		}
	} 
	else
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1800DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						wRTempData =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+0); 
						wRTempData+=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*6+1)<<8; 
						wGTempData =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+2);
						wGTempData+=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*6+3)<<8; 
						wBTempData =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+4); 
						wBTempData+=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*6+5)<<8;
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wBTempData+131072]);
						i++;
						wRTempData =*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+0);
						wRTempData+=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*6+1)<<8;
						wGTempData =*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+2);
						wGTempData+=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*6+3)<<8;
						wBTempData =*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+4);
						wBTempData+=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*6+5)<<8; 
						*(lpLine+i*6+4)=LOBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+5)=HIBYTE(g_pGammaTable[wRTempData]);
						*(lpLine+i*6+2)=LOBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+3)=HIBYTE(g_pGammaTable[wGTempData+65536]);
						*(lpLine+i*6+0)=LOBYTE(g_pGammaTable[wBTempData+131072]);
						*(lpLine+i*6+1)=HIBYTE(g_pGammaTable[wBTempData+131072]);
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;
				
			}
			if(g_isCanceled)
			{	
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
	
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb48BitLine1800DPI: thread exit");

				break;
			}
		}
	}
	*wLinesCount = TotalXferLines;

	g_isScanning=FALSE;
	return TRUE;
}

BOOL MustScanner_GetRgb24BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{

	WORD wWantedTotalLines;
	WORD TotalXferLines;
	WORD wRLinePos=0;
	WORD wGLinePos=0;
	WORD wBLinePos=0;
	BYTE byRed;
	BYTE byGreen;
	BYTE byBlue;
	BYTE bNextPixel=0;
	BOOL bImagePixelAvg900dpi;
	WORD i;

	WORD tempR,tempG,tempB;
	
	g_isCanceled=FALSE;
	g_isScanning=TRUE;

	wWantedTotalLines=*wLinesCount;
	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: get wWantedTotalLines= %d",wWantedTotalLines);

	TotalXferLines=0;

	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: call in");
	
	bImagePixelAvg900dpi = OptionReadInt("[spicall]","ImagePixelAvg900dpi",0,INIFILENAME);	

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread create");

		g_bFirstReadImage=FALSE;

	}	

	if(!isOrderInvert)
	{		
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: !isOrderInvert");

		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{

			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{	
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					if(bImagePixelAvg900dpi)
					{	
						byRed   =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*3+0);
						bNextPixel = *(g_lpReadImageHead+wRLinePos*g_BytesPerRow+(i+1)*3+0); 
						if(abs(byRed - bNextPixel) < AVG_THRESHOLD)
							byRed = (byRed + bNextPixel) >> 1;
					
	
						byGreen =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*3+1);
						bNextPixel=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+(i+1)*3+1);
						if(abs(byGreen - bNextPixel) < AVG_THRESHOLD)
							byGreen = (byGreen + bNextPixel) >> 1;
					
						byBlue  =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*3+2);
						bNextPixel=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+(i+1)*3+2);
						if(abs(byBlue - bNextPixel) < AVG_THRESHOLD)
							byBlue = (byBlue + bNextPixel) >> 1;
					}
					else
					{
						byRed=*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*3+0);
						byGreen=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*3+1);
						byBlue=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*3+2);
					}					

					tempR = (WORD)((byRed<<4)  | QBET4(byBlue,byGreen));
					tempG = (WORD)((byGreen<<4)| QBET4(byRed,byBlue));
					tempB = (WORD)((byBlue<<4) | QBET4(byGreen,byRed));   				    

					*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable    +tempR));
					*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+tempG));
					*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable+8192+tempB));
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;
					
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_dwTotalTotalXferLines=%d,g_SWHeight=%d",g_dwTotalTotalXferLines,g_SWHeight);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_SWBytesPerRow=%d",g_SWBytesPerRow);
				
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				break;
			}
		}
	}
	else
	{
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: isOrderInvert is TRUE");

		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
		
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				wRLinePos=g_wtheReadyLines%g_wMaxScanLines;
				wGLinePos=(g_wtheReadyLines-g_wLineDistance)%g_wMaxScanLines;
				wBLinePos=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; i++)
				{
					if(bImagePixelAvg900dpi)
					{	
						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: bImagePixelAvg900dpi is TRUE");


						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byRed");
						byRed   =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*3+0);
						bNextPixel = *(g_lpReadImageHead+wRLinePos*g_BytesPerRow+(i+1)*3+0);
						if(abs(byRed - bNextPixel) < AVG_THRESHOLD)
							byRed = (byRed + bNextPixel) >> 1;

						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byGreen");

						byGreen =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*3+1);
						bNextPixel=*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+(i+1)*3+1); 
						if(abs(byGreen - bNextPixel) < AVG_THRESHOLD)
							byGreen = (byGreen + bNextPixel) >> 1;
			
						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byBlue");
						
						byBlue  =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*3+2);
						bNextPixel=*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+(i+1)*3+2); 
						if(abs(byBlue - bNextPixel) < AVG_THRESHOLD)
							byBlue = (byBlue + bNextPixel) >> 1;
					}
					else
					{		
						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byRed");						

						byRed   =*(g_lpReadImageHead+wRLinePos*g_BytesPerRow+i*3+0);
			
						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byGreen");						
						byGreen =*(g_lpReadImageHead+wGLinePos*g_BytesPerRow+i*3+1);
						
						XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before byBlue");
						byBlue  =*(g_lpReadImageHead+wBLinePos*g_BytesPerRow+i*3+2);
					}					
						
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: before set lpLine");
					XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: i=%d",i);

					*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
					*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
					*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
					
					
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;
								
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_dwTotalTotalXferLines=%d,g_SWHeight=%d",g_dwTotalTotalXferLines,g_SWHeight);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: g_SWBytesPerRow=%d",g_SWBytesPerRow);

			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine: thread exit");

				break;
			}
		} 
	}

	*wLinesCount = TotalXferLines;

	g_isScanning=FALSE;
	return TRUE;
}

BOOL MustScanner_GetRgb24BitLine1800DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	BYTE* lpTemp;
	WORD wWantedTotalLines;
	WORD TotalXferLines;
	WORD wRLinePosOdd=0;
	WORD wGLinePosOdd=0;
	WORD wBLinePosOdd=0;
	WORD wRLinePosEven=0;
	WORD wGLinePosEven=0;
	WORD wBLinePosEven=0;
	BYTE byRed;
	BYTE byGreen;
	BYTE byBlue;
	BYTE bNextPixel=0;
	BOOL isSoftwareCalibration;	
	WORD i;

	BOOL bImagePixelAvg;



	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	TotalXferLines=0;
	wWantedTotalLines=*wLinesCount;
	lpTemp = lpLine;


	XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: call in");

	bImagePixelAvg= OptionReadInt("[spicall]","ImagePixelAvg",0,INIFILENAME);


	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: thread create");

		g_bFirstReadImage=FALSE;
	}


#ifdef HARDWARE_SHADING_SWITCH_ON
	if(g_Width > HARDWARE_SHADING_WIDTH * 1800)
		isSoftwareCalibration = TRUE;
	else 
		isSoftwareCalibration = FALSE;
#else
	isSoftwareCalibration = TRUE;
#endif

	if(isSoftwareCalibration == TRUE)
	{
		XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI,isSoftwareCalibration==TRUE");
	}

	if(!isOrderInvert)
	{
		for ( ; TotalXferLines<wWantedTotalLines; )
		{

			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_Height=%d",g_Height);

				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				if(isSoftwareCalibration)
				{					
					WORD wDoShadingLine;
					wDoShadingLine = (WORD)g_dwScannedTotalLines - g_wReadyShadingLine;
					
					if(wDoShadingLine)  
					{
						if((g_wStartShadingLinePos + wDoShadingLine) >= g_wMaxScanLines)  
						{						
							WORD wFirstShadingLines;
							
							wFirstShadingLines = g_wMaxScanLines - g_wStartShadingLinePos;
							
							wDoShadingLine = wDoShadingLine - wFirstShadingLines;
							
							Asic_DoSoftwareShadingRGB24Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wFirstShadingLines);
							
							g_wReadyShadingLine += wFirstShadingLines;
							g_wStartShadingLinePos = 0;
							
							Asic_DoSoftwareShadingRGB24Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						}
						else
						{
							Asic_DoSoftwareShadingRGB24Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						}
						g_wReadyShadingLine += wDoShadingLine;
						g_wStartShadingLinePos += wDoShadingLine;
					}
								
				}				
				
				wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_Width; )
				//for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						if(bImagePixelAvg)
						{
							byRed   =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*3+0);
							bNextPixel=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*3+0); 
							if(abs(byRed - bNextPixel) < AVG_THRESHOLD)
								byRed = (byRed + bNextPixel) >> 1;
							
							byGreen =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*3+1);
							bNextPixel=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*3+1); 
							if(abs(byGreen - bNextPixel) < AVG_THRESHOLD)
								byGreen = (byGreen + bNextPixel) >> 1;
							
							byBlue  =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*3+2);
							bNextPixel=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*3+2);
							if(abs(byBlue - bNextPixel) < AVG_THRESHOLD)
								byBlue = (byBlue + bNextPixel) >> 1;
						}
						else
						{
							byRed   =((*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*3+0)));
							byGreen =((*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*3+1)));
							byBlue  =((*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*3+2)));
						}
						

#ifndef _NOT_DO_GAMMA
						
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
					    *(lpLine+i*3+0) = byRed;
					    *(lpLine+i*3+1) = byGreen;
					    *(lpLine+i*3+2) = byBlue;
#endif
						i++;

						if(bImagePixelAvg)
						{
							byRed	=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*3+0);
							bNextPixel=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*3+0);
							if(abs(byRed - bNextPixel) < AVG_THRESHOLD)
								byRed = (byRed + bNextPixel) >> 1;
							
							byGreen	=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*3+1);
							bNextPixel=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*3+1);
							if(abs(byGreen - bNextPixel) < AVG_THRESHOLD)
								byGreen = (byGreen + bNextPixel) >> 1;
							
							byBlue	=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*3+2);
							bNextPixel=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*3+2);
							if(abs(byBlue - bNextPixel) < AVG_THRESHOLD)
								byBlue = (byBlue + bNextPixel) >> 1;
						}
						else
						{
							byRed	=((*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*3+0)));
							byGreen	=((*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*3+1)));
							byBlue	=((*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*3+2)));
						}

#ifndef _NOT_DO_GAMMA
						
						*(lpLine+i*3+0) =(unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) =(unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+2) =(unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
					    *(lpLine+i*3+0) = byRed;
					    *(lpLine+i*3+1) = byGreen;
					    *(lpLine+i*3+2) = byBlue;
#endif
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_Height=%d",g_Height);
			
			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: thread exit");

				break;
			}
					
		} 
	}
	else
	{
		for ( ; TotalXferLines<wWantedTotalLines; ) 
		{
			if(g_dwTotalTotalXferLines>=g_SWHeight)
			{
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_Height=%d",g_Height);

				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: thread exit");

				*wLinesCount = TotalXferLines;
				g_isScanning=FALSE;
				return TRUE;
			}

			if(g_dwScannedTotalLines > g_wtheReadyLines)
			{
				
				if(isSoftwareCalibration)
				{
					
					
					WORD wDoShadingLine;
					wDoShadingLine = (WORD)g_dwScannedTotalLines - g_wReadyShadingLine;					
					
					if(wDoShadingLine)  
					{						
						if((g_wStartShadingLinePos + wDoShadingLine) >= g_wMaxScanLines) 
						{												
							WORD wFirstShadingLines;							
							wFirstShadingLines = g_wMaxScanLines - g_wStartShadingLinePos;							
							wDoShadingLine = wDoShadingLine - wFirstShadingLines;
							
							Asic_DoSoftwareShadingRGB24Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wFirstShadingLines);
							
							g_wReadyShadingLine += wFirstShadingLines;
							g_wStartShadingLinePos = 0;
							
							Asic_DoSoftwareShadingRGB24Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						}
						else
						{
							Asic_DoSoftwareShadingRGB24Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						}
						g_wReadyShadingLine += wDoShadingLine;
						g_wStartShadingLinePos += wDoShadingLine;
					}
					
				}				
				
				wRLinePosOdd=(g_wtheReadyLines                  -g_wPixelDistance)%g_wMaxScanLines;
				wGLinePosOdd=(g_wtheReadyLines-g_wLineDistance  -g_wPixelDistance)%g_wMaxScanLines;
				wBLinePosOdd=(g_wtheReadyLines-g_wLineDistance*2-g_wPixelDistance)%g_wMaxScanLines;
				wRLinePosEven=(g_wtheReadyLines                  )%g_wMaxScanLines;
				wGLinePosEven=(g_wtheReadyLines-g_wLineDistance  )%g_wMaxScanLines;
				wBLinePosEven=(g_wtheReadyLines-g_wLineDistance*2)%g_wMaxScanLines;
				
				for(i=0; i<g_SWWidth; )
				{
					if((i+1)!=g_SWWidth) 
					{
						if(bImagePixelAvg)
						{
							byRed   =*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*3+0);
							bNextPixel=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+(i+1)*3+0);
							if(abs(byRed - bNextPixel) < AVG_THRESHOLD)
								byRed = (byRed + bNextPixel) >> 1;
							
							byGreen =*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*3+1);
							bNextPixel=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+(i+1)*3+1);
							if(abs(byGreen - bNextPixel) < AVG_THRESHOLD)
								byGreen = (byGreen + bNextPixel) >> 1;
							
							byBlue  =*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*3+2);
							bNextPixel=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+(i+1)*3+2); 
							if(abs(byBlue - bNextPixel) < AVG_THRESHOLD)
								byBlue = (byBlue + bNextPixel) >> 1;
						}
						else
						{
							byRed   =((*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+i*3+0)));
							byGreen =((*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+i*3+1)));
							byBlue  =((*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+i*3+2)));
						}


#ifndef _NOT_DO_GAMMA
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
						*(lpLine+i*3+0) = byRed;
						*(lpLine+i*3+1) = byGreen;
						*(lpLine+i*3+2) = byBlue;
#endif
						i++;

						if(bImagePixelAvg)
						{
							byRed	=*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*3+0);
							bNextPixel=*(g_lpReadImageHead+wRLinePosOdd*g_BytesPerRow+(i+1)*3+0); 
							if(abs(byRed - bNextPixel) < AVG_THRESHOLD)
								byRed = (byRed + bNextPixel) >> 1;
							
							byGreen	=*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*3+1);
							bNextPixel=*(g_lpReadImageHead+wGLinePosOdd*g_BytesPerRow+(i+1)*3+1); 
							if(abs(byGreen - bNextPixel) < AVG_THRESHOLD)
								byGreen = (byGreen + bNextPixel) >> 1;
							
							byBlue	=*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*3+2);
							bNextPixel=*(g_lpReadImageHead+wBLinePosOdd*g_BytesPerRow+(i+1)*3+2);
							if(abs(byBlue - bNextPixel) < AVG_THRESHOLD)
								byBlue = (byBlue + bNextPixel) >> 1;
						}
						else
						{
							byRed	=((*(g_lpReadImageHead+wRLinePosEven*g_BytesPerRow+i*3+0)));
							byGreen	=((*(g_lpReadImageHead+wGLinePosEven*g_BytesPerRow+i*3+1)));
							byBlue	=((*(g_lpReadImageHead+wBLinePosEven*g_BytesPerRow+i*3+2)));
						}
						
#ifndef _NOT_DO_GAMMA
						
						*(lpLine+i*3+2) = (unsigned char)(*(g_pGammaTable    +(WORD)((byRed<<4)  | QBET4(byBlue,byGreen))));
						*(lpLine+i*3+1) = (unsigned char)(*(g_pGammaTable+4096+(WORD)((byGreen<<4)| QBET4(byRed,byBlue))));
						*(lpLine+i*3+0) = (unsigned char)(*(g_pGammaTable+8192+(WORD)((byBlue<<4) | QBET4(byGreen,byRed))));
#else
						*(lpLine+i*3+0) = byRed;
						*(lpLine+i*3+1) = byGreen;
						*(lpLine+i*3+2) = byBlue;
#endif
						i++;
					}
				}					
				
				TotalXferLines++;
				g_dwTotalTotalXferLines++;
				lpLine+=g_SWBytesPerRow;
				g_wtheReadyLines++;

				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_dwTotalTotalXferLines=%d",g_dwTotalTotalXferLines);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: g_Height=%d",g_Height);

			}
			if(g_isCanceled)
			{
				pthread_cancel(g_threadid_readimage);
				pthread_join(g_threadid_readimage,NULL);
				XDBG(DEBUG_SPICALL,"MustScanner_GetRgb24BitLine1800DPI: thread exit");

				break;
			}
		} 
	}		
	
	if(FALSE==g_isCanceled)
	{
		if(g_bSharpen)
			MustScanner_CrossSharpen(lpTemp,wWantedTotalLines,g_SWBytesPerRow,2);
	}
	

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;
	return TRUE;
	
}

BOOL MustScanner_GetMono16BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	WORD wTempData;
	
	WORD wLinePos=0;
	WORD i;


	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;	
	

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: call in");

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}
	
	for ( ; TotalXferLines<wWantedTotalLines; ) 
	{
		
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(g_dwScannedTotalLines > g_wtheReadyLines)
		{
			wLinePos=g_wtheReadyLines%g_wMaxScanLines;
			
			for(i=0; i<g_Width; i++)
			{
				wTempData =*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i*2+0);
				wTempData+=*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i*2+1)<<8;
				*(lpLine+i*2+0)=LOBYTE(g_pGammaTable[wTempData]);
				*(lpLine+i*2+1)=HIBYTE(g_pGammaTable[wTempData]);
			}					
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow;
			g_wtheReadyLines++;
						
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine: thread exit");

			break;
		}
	}	

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;
	return TRUE;

}

BOOL MustScanner_GetMono16BitLine1800DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	DWORD dwTempData;
	WORD wLinePosOdd=0;
	WORD wLinePosEven=0;	
	BOOL isSoftwareCalibration;
	WORD i;

	BOOL bImagePixelAvg;


	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	
	
	

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1800DPI: call in");

	bImagePixelAvg = OptionReadInt("[spicall]","ImagePixelAvg",0,INIFILENAME);


	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1800DPI: thread create");
		g_bFirstReadImage=FALSE;
	}


#ifdef HARDWARE_SHADING_SWITCH_ON
	if(g_Width > HARDWARE_SHADING_WIDTH * 1800)
		isSoftwareCalibration = TRUE;
	else 
		isSoftwareCalibration = FALSE;
#else
	isSoftwareCalibration = TRUE;
#endif

	for ( ; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1800DPI: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(g_dwScannedTotalLines > g_wtheReadyLines)
		{
			if(isSoftwareCalibration)
			{				
				WORD wDoShadingLine;
				wDoShadingLine = (WORD)g_dwScannedTotalLines - g_wReadyShadingLine;
								
				if(wDoShadingLine)  
				{
					
					if((g_wStartShadingLinePos + wDoShadingLine) >= g_wMaxScanLines)   //buffer over flow
					{
						
						WORD wFirstShadingLines;
						
						wFirstShadingLines = g_wMaxScanLines - g_wStartShadingLinePos;
						
						wDoShadingLine = wDoShadingLine - wFirstShadingLines;
						
						Asic_DoSoftwareShadingGray16Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wFirstShadingLines);
						
						g_wReadyShadingLine += wFirstShadingLines;
						g_wStartShadingLinePos = 0;
						
						Asic_DoSoftwareShadingGray16Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
						
					}
					else
					{
						Asic_DoSoftwareShadingGray16Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
					}
					g_wReadyShadingLine += wDoShadingLine;
					g_wStartShadingLinePos += wDoShadingLine;
				}
				
			}
			
		
			wLinePosOdd=(g_wtheReadyLines)%g_wMaxScanLines;
			wLinePosEven=(g_wtheReadyLines)%g_wMaxScanLines;
			
			for(i=0; i<g_Width; )
			{
				if((i+1)!=g_SWWidth) 
				{
					if(bImagePixelAvg)
					{
						dwTempData =(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i*2+0)); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i*2+1)<<8); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+(i+1)*2+0)); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+(i+1)*2+1)<<8); 
						dwTempData=g_pGammaTable[dwTempData>>1];
						*(lpLine+i*2+0)=LOBYTE((WORD)dwTempData);
						*(lpLine+i*2+1)=HIBYTE((WORD)dwTempData);
						i++;
						dwTempData =(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i*2+0)); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i*2+1)<<8); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)*2+0)); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)*2+1)<<8); 
						dwTempData=g_pGammaTable[dwTempData>>1];
						*(lpLine+i*2+0)=LOBYTE((WORD)dwTempData);
						*(lpLine+i*2+1)=HIBYTE((WORD)dwTempData);
						i++;
					}
					else
					{
						dwTempData=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)*2+0)); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)*2+1)<<8); 
						dwTempData=g_pGammaTable[dwTempData];
						*(lpLine+i*2+0)=LOBYTE((WORD)dwTempData);
						*(lpLine+i*2+1)=HIBYTE((WORD)dwTempData);
						i++;
						dwTempData =(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i*2+0)); 
						dwTempData+=(DWORD)(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i*2+1)<<8); 
						dwTempData=g_pGammaTable[dwTempData];
						*(lpLine+i*2+0)=LOBYTE((WORD)dwTempData);
						*(lpLine+i*2+1)=HIBYTE((WORD)dwTempData);
						i++;
					}
				}
			}					
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow;
			g_wtheReadyLines++;
			
			
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono16BitLine1800DPI: thread exit");

			break;
		}
	}
	

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;
	return TRUE;
	
}

BOOL MustScanner_GetMono8BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	WORD i;
	WORD wLinePos=0;

	
	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;

	
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: call in");


	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}	
	
	for ( ; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(g_dwScannedTotalLines > g_wtheReadyLines)
		{
			wLinePos=g_wtheReadyLines%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; i++)
			{
				*(lpLine+i) = (BYTE)*(g_pGammaTable+(WORD)((*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i)<<4) 
					| (rand()&0x0f)));
			}				
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow;
			g_wtheReadyLines++;
			
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine: thread exit");

			break;
		}
	}
	
	*wLinesCount = TotalXferLines;

	g_isScanning=FALSE;

	return TRUE;
}

BOOL MustScanner_GetMono8BitLine1800DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	BYTE* lpTemp;		
	WORD wWantedTotalLines;
	WORD TotalXferLines;  
	
	WORD wLinePosOdd=0;
	WORD wLinePosEven=0;
	BYTE byGray;	
	BOOL isSoftwareCalibration;	
    WORD i;
	BYTE bNextPixel = 0;	

	BOOL bImagePixelAvg;	


	TotalXferLines=0;
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	lpTemp=lpLine;

	XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1800DPI: call in");

	bImagePixelAvg= OptionReadInt("[spicall]","ImagePixelAvg",0,INIFILENAME);

	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1800DPI: thread create");
		g_bFirstReadImage=FALSE;
	}
	
#ifdef HARDWARE_SHADING_SWITCH_ON
	if(g_Width > HARDWARE_SHADING_WIDTH * 1800)
		isSoftwareCalibration = TRUE;
	else 
		isSoftwareCalibration = FALSE;
#else
	isSoftwareCalibration = TRUE;
#endif

	for (;TotalXferLines<wWantedTotalLines;) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1800DPI: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(g_dwScannedTotalLines > g_wtheReadyLines)
		{
			if(isSoftwareCalibration)
			{
				WORD wDoShadingLine;
				wDoShadingLine = (WORD)g_dwScannedTotalLines - g_wReadyShadingLine;
				if(wDoShadingLine)  
				{
					
					if((g_wStartShadingLinePos + wDoShadingLine) >= g_wMaxScanLines) 
					{
						
						WORD wFirstShadingLines;
						
						wFirstShadingLines = g_wMaxScanLines - g_wStartShadingLinePos;
						wDoShadingLine = wDoShadingLine - wFirstShadingLines;
						
						Asic_DoSoftwareShadingGray8Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wFirstShadingLines);
						
						g_wReadyShadingLine += wFirstShadingLines;
						g_wStartShadingLinePos = 0;
						
						Asic_DoSoftwareShadingGray8Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
					}
					else
					{
						Asic_DoSoftwareShadingGray8Bit(&g_chip,g_lpReadImageHead, g_Width,  g_wStartShadingLinePos, wDoShadingLine);
					}
					g_wReadyShadingLine += wDoShadingLine;
					g_wStartShadingLinePos += wDoShadingLine;
				}
			}
			wLinePosOdd=(g_wtheReadyLines-g_wPixelDistance   )%g_wMaxScanLines;
			wLinePosEven=(g_wtheReadyLines)%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; )
			{
				if((i+1)!=g_Width) 
				{
					
					if(bImagePixelAvg)
					{
						byGray=*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i);
						bNextPixel=*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+(i+1)); 
						if(abs(byGray - bNextPixel) < AVG_THRESHOLD)
							byGray = (byGray + bNextPixel) >> 1;
						
					}
					else
					{
						byGray=(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i));
					}
					
					*(lpLine+i) = (BYTE)*(g_pGammaTable+( byGray<<4 | (rand()&0x0f)));
					
					i++;
					
					if(bImagePixelAvg)
					{
						byGray=*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i);
						bNextPixel=*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+(i+1)); 
						if(abs(byGray - bNextPixel) < AVG_THRESHOLD)
							byGray = (byGray + bNextPixel) >> 1;
					}
					else
					{
						byGray=(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i));
					}
					
					*(lpLine+i) = (BYTE)*(g_pGammaTable+( byGray<<4 | (rand()&0x0f)));
					i++;
				}
			}				
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow;
			g_wtheReadyLines++;
		}
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono8BitLine1800DPI: thread exit");

			break;
		}
	}
	

	if(FALSE==g_isCanceled)
	{
		if(g_bSharpen)
			MustScanner_CrossSharpen(lpTemp,wWantedTotalLines,g_SWBytesPerRow,1);
	}

	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;
	return TRUE;
	
}

BOOL MustScanner_GetMono1BitLine(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	WORD wWantedTotalLines;
	WORD TotalXferLines;  
	WORD wLinePos;
	WORD i;


	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: call in");



	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: thread create");
		g_bFirstReadImage=FALSE;
	}
	
	memset(lpLine,0,wWantedTotalLines*g_SWWidth/8);
	
	for ( TotalXferLines=0; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}

		if(g_dwScannedTotalLines > g_wtheReadyLines)
		{
			wLinePos=g_wtheReadyLines%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; i++ )
			{
				
				if(*(g_lpReadImageHead+wLinePos*g_BytesPerRow+i ) > g_wLineartThreshold) 
				{

					*(lpLine+i/8)+=(0x80 >> (i%8));
				}
			}
		
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=(g_SWBytesPerRow/8);
			g_wtheReadyLines++;
		
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine: thread exit");

			break;
		}
	}


	*wLinesCount = TotalXferLines;
	g_isScanning=FALSE;
	return TRUE;

}

BOOL MustScanner_GetMono1BitLine1800DPI(BYTE* lpLine,BOOL isOrderInvert,WORD *wLinesCount)
{
	WORD wWantedTotalLines;
	WORD TotalXferLines;   
	WORD i;	
	WORD wLinePosOdd;
	WORD wLinePosEven;

	
	g_isCanceled=FALSE;
	g_isScanning=TRUE;
	wWantedTotalLines=*wLinesCount;
	
	XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1800DPI: call in");



	if(g_bFirstReadImage)
	{
		pthread_create(&g_threadid_readimage,NULL,MustScanner_ReadDataFromScanner,NULL);
		XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1800DPI: thread create");
		g_bFirstReadImage=FALSE;
	}
	
	memset(lpLine,0,wWantedTotalLines*g_SWWidth/8);
	
	for ( TotalXferLines=0; TotalXferLines<wWantedTotalLines; ) 
	{
		if(g_dwTotalTotalXferLines>=g_SWHeight)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1800DPI: thread exit");

			*wLinesCount = TotalXferLines;
			g_isScanning=FALSE;
			return TRUE;
		}
		
		if(g_dwScannedTotalLines > g_wtheReadyLines)
		{
			wLinePosOdd =(g_wtheReadyLines  )%g_wMaxScanLines;
			wLinePosEven=(g_wtheReadyLines-g_wPixelDistance)%g_wMaxScanLines;
			
			for(i=0; i<g_SWWidth; )
			{
				if((i+1)!=g_SWWidth) 
				{
					if(*(g_lpReadImageHead+wLinePosOdd*g_BytesPerRow+i) > g_wLineartThreshold) 
						*(lpLine+i/8)+=(0x80 >> (i%8)); 
					i++;
					if(*(g_lpReadImageHead+wLinePosEven*g_BytesPerRow+i) > g_wLineartThreshold) 
						*(lpLine+i/8)+=(0x80 >> (i%8)); 
					i++;
				}
			}					
			
			TotalXferLines++;
			g_dwTotalTotalXferLines++;
			lpLine+=g_SWBytesPerRow/8;
			g_wtheReadyLines++;
			
		} 
		if(g_isCanceled)
		{
			pthread_cancel(g_threadid_readimage);
			pthread_join(g_threadid_readimage,NULL);
			XDBG(DEBUG_SPICALL,"MustScanner_GetMono1BitLine1800DPI: thread exit");

			break;
		}
	}

	*wLinesCount = TotalXferLines;

	g_isScanning=FALSE;
	return TRUE;
}



void MustScanner_CrossSharpen(BYTE* lpLine,WORD wLineCount, DWORD LineBytes, BYTE ScanMode)
{
	USHORT Row,Col;
	USHORT Lines;
	UINT   Index;
	LPBYTE ImgAddr;
	short  TmpColor1,TmpColor2;
    float  NormalInfo[1020];
	int i;

	XDBG(DEBUG_SPICALL,"MustScanner_CrossSharpen: call in");
	
	for(i=0;i<1020;i++)
	{	
		
		if(i<40)
		{
			NormalInfo[i] = (float)(0.5-0.0138); 
		}
		else if(i<80)
		{
			NormalInfo[i]= (float)(0.5-0.0712);  
		}
		else if(i<120)
		{
			NormalInfo[i]= (float)(0.5-0.1678);  
		}
		else if(i<160)
		{
			NormalInfo[i]= (float)(0.5-0.2397);  
		}
		else if(i<200)
		{
			NormalInfo[i]= (float)(0.5-0.2311);  
		}
		else if(i<240)
		{
			NormalInfo[i]= (float)(0.5-0.1585);  
		}
		else if(i<280)
		{
			NormalInfo[i]= (float)(0.5-0.0792);  
		}
		else if(i<320)
		{
			NormalInfo[i]= (float)(0.5-0.0291);  
		}
		else if(i<480)
		{
			NormalInfo[i]= (float)(0.5-0.0078);  
		}
		else if(i<520)
		{
			NormalInfo[i]= (float)(0.5-0.0015);
		}
		else 
		{
			NormalInfo[i]=0;
		}
	}
	Lines  =wLineCount;
    ImgAddr=lpLine;
	
	if(ScanMode==1)
	{
		unsigned char *ptColor=(unsigned char*)malloc(LineBytes*3);
		unsigned char *ptColor1=ptColor; 
        unsigned char *ptColor2=&ptColor[LineBytes]; 
		unsigned char *ptColor3=&ptColor[LineBytes*2];
		
		for(Col=0;Col< LineBytes;Col++)
		{
			*(ptColor2+Col)=(BYTE)(*( ImgAddr+Col));				
			*(ptColor3+Col)=(BYTE)(*( ImgAddr+LineBytes+Col));			
		}
		
		TmpColor1=( (*(ptColor2)- *(ptColor2+1))+(*(ptColor2)- *(ptColor3)));
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		TmpColor2=(BYTE)(*(ImgAddr));
		if( (TmpColor2+TmpColor1)<=0)
		{
			*(ImgAddr)=0;
		}
		else if((TmpColor2+TmpColor1) >=255)
		{
			*(ImgAddr)=255;
		}
		else
		{
			*(ImgAddr)+=TmpColor1;
		}
		
		TmpColor1= ((*(ptColor2+LineBytes-1) - *(ptColor2+LineBytes-2))+
			         (*(ptColor2+LineBytes-1) - *(ptColor3+LineBytes-1)) ) ;
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		TmpColor2=(BYTE)(*( ImgAddr+LineBytes-1));
		if( ( TmpColor2+TmpColor1 )<=0  )
		{
			*(ImgAddr+LineBytes-1)=0;
		}
		else if( (TmpColor2+TmpColor1) >=255 )
		{
			*(ImgAddr+LineBytes-1)=255;
		}
		else
		{
			*(ImgAddr+LineBytes-1)+=TmpColor1;
		}
		
		for(Col=1;Col< (LineBytes-1) ;Col++)
		{
			TmpColor1= ((*(ptColor2+Col)-*(ptColor2+Col-1))+
				(*(ptColor2+Col)-*(ptColor2+Col+1))+			        
				(*(ptColor2+Col)-*(ptColor3+Col  )));
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			TmpColor2=(BYTE)(*( ImgAddr+Col));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*( ImgAddr+Col)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*( ImgAddr+Col)=255;
			}
			else
			{
				*( ImgAddr+Col)+=TmpColor1;
			}
        }

		for(Row=1;Row< (Lines-1);Row++)
		{
			ptColor1=ptColor2;
			ptColor2=ptColor3;
			
			for(Col=0;Col< LineBytes;Col++)
			{
				*(ptColor3+Col)=(BYTE)(*(ImgAddr+LineBytes*(Row+1)+Col));
			}

			TmpColor1=( (*(ptColor2)- *(ptColor2+1))+
				(*(ptColor2)- *(ptColor1))  +
				(*(ptColor2)- *(ptColor3))    );
			
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+Row*LineBytes)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+Row*LineBytes)=255;
			}
			else
			{
				*(ImgAddr+Row*LineBytes)+=TmpColor1;
			}
			
			TmpColor1=( (*(ptColor2+LineBytes-1)- *(ptColor2+LineBytes-2))+
				(*(ptColor2+LineBytes-1)- *(ptColor1+LineBytes-1))  +
				(*(ptColor2+LineBytes-1)- *(ptColor3+LineBytes-1))    );
			
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+(Row+1)*LineBytes-1));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+(Row+1)*LineBytes-1)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Row+1)*LineBytes-1)=255;
			}
			else
			{
				*(ImgAddr+(Row+1)*LineBytes-1)+=TmpColor1;
			}
			
			for(Col=1;Col<(LineBytes-1);Col++)
			{
				TmpColor1=( (*(ptColor2+Col)- *(ptColor2+Col-1))+
					(*(ptColor2+Col)- *(ptColor2+Col+1))+
					(*(ptColor2+Col)- *(ptColor1+Col))  +
					(*(ptColor2+Col)- *(ptColor3+Col))    ) ;
				
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes+Col));
				if( (TmpColor2+TmpColor1)<=0)
				{
					*(ImgAddr+Row*LineBytes+Col)=0;
				}
				else if((TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+Row*LineBytes+Col)=255;
				}
				else
				{
					*(ImgAddr+Row*LineBytes+Col)+=TmpColor1;
				}				
			}
			
		}

		TmpColor1=( (*(ptColor3)- *(ptColor3+1))+
			(*(ptColor3)- *(ptColor2  )) );
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes));
		if( (TmpColor2+TmpColor1)<=0)
		{
			*(ImgAddr+(Lines-1)*LineBytes)=0;
		}
		else if((TmpColor2+TmpColor1) >=255 )
		{
			*(ImgAddr+(Lines-1)*LineBytes)=255;
		}
		else
		{
			*(ImgAddr+(Lines-1)*LineBytes)+=TmpColor1;
		}
		
		TmpColor1= ((*(ptColor3+LineBytes-1) - *(ptColor3+LineBytes-2))+
			(*(ptColor3+LineBytes-1) - *(ptColor3+LineBytes-1))  ) ;
		TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
		
		TmpColor2=(BYTE)(*( ImgAddr+(Lines*LineBytes)-1));
		if( ( TmpColor2+TmpColor1 )<=0  )
		{
			*(ImgAddr+(Lines*LineBytes)-1)=0;
		}
		else if( (TmpColor2+TmpColor1) >=255 )
		{
			*(ImgAddr+(Lines*LineBytes)-1)=255;
		}
		else
		{
			*(ImgAddr+(Lines*LineBytes)-1)+=TmpColor1;
		}
		
		for(Col=1;Col< (LineBytes-1) ;Col++)
		{
			TmpColor1= (  (*(ptColor3+Col)-*(ptColor3+Col-1))+
				(*(ptColor3+Col)-*(ptColor3+Col+1))+			        
				(*(ptColor3+Col)-*(ptColor2+Col))    ) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes+Col));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*(ImgAddr+(Lines-1)*LineBytes+Col)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Lines-1)*LineBytes+Col)=255;
			}
			else
			{
				*(ImgAddr+(Lines-1)*LineBytes+Col)+=TmpColor1;
			}
        }		
		
		free(ptColor);
		
	}
	else if(ScanMode==2)
	{ 	
		unsigned char *ptColor=(unsigned char*)malloc(LineBytes*3);
		unsigned char *ptColor1=ptColor;            
        unsigned char *ptColor2=&ptColor[LineBytes]; 
		unsigned char *ptColor3=&ptColor[LineBytes*2];
		
		for(Col=0;Col< LineBytes;Col++)
		{
			*(ptColor2+Col)=(BYTE)(*( ImgAddr+Col));				
			*(ptColor3+Col)=(BYTE)(*( ImgAddr+LineBytes+Col));			
		}

		for(Index=0;Index<3;Index++)
		{
			TmpColor1=(( (*(ptColor2+Index)- *(ptColor2+3+Index))+
				(*(ptColor2+Index)- *(ptColor3+Index  ))   )) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+Index));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+Index)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+Index)=255;
			}
			else
			{
				*(ImgAddr+Index)+=TmpColor1;
			}

			TmpColor1=( ((*(ptColor2+(LineBytes-3)+Index ) - *(ptColor2+(LineBytes-6)+Index))+
				(*(ptColor2+(LineBytes-3)+Index ) - *(ptColor3+(LineBytes-3)+Index))  ) ) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*( ImgAddr+LineBytes-3+Index));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*(ImgAddr+LineBytes-3+Index)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+LineBytes-3+Index)=255;
			}
			else
			{
				*(ImgAddr+LineBytes-3+Index)+=TmpColor1;
			}
		}

		for(Col=3;Col< (LineBytes-1) ;Col+=3)
		{
			for(Index=0;Index<3;Index++)
			{
				TmpColor1=( (  (*(ptColor2+Col+Index)-*(ptColor2+Col-3+Index))+
					(*(ptColor2+Col+Index)-*(ptColor2+Col+3+Index))+			        
					(*(ptColor2+Col+Index)-*(ptColor3+Col+Index  ))  ) ) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*( ImgAddr+Col+Index));
				if( ( TmpColor2+TmpColor1 )<=0  )
				{
					*( ImgAddr+Col+Index)=0;
				}
				else if( (TmpColor2+TmpColor1) >=255 )
				{
					*( ImgAddr+Col+Index)=255;
				}
				else
				{
					*( ImgAddr+Col+Index)+=TmpColor1;
				}
			}	
        }

		for(Row=1;Row< (Lines-1);Row++)
		{
			ptColor1=ptColor2;
			ptColor2=ptColor3;
			
			for(Col=0;Col< LineBytes;Col++)
			{
				*(ptColor3+Col)=(BYTE)(*(ImgAddr+LineBytes*(Row+1)+Col));
			}
			
			for(Index=0;Index<3;Index++)
			{
				TmpColor1=(( (*(ptColor2+Index)- *(ptColor2+3+Index))+
					(*(ptColor2+Index)- *(ptColor1+Index  ))+
					(*(ptColor2+Index)- *(ptColor3+Index  ))  )) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes+Index));
				if( (TmpColor2+TmpColor1)<=0)
				{
					*(ImgAddr+Row*LineBytes+Index)=0;
				}
				else if((TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+Row*LineBytes+Index)=255;
				}
				else
				{
					*(ImgAddr+Row*LineBytes+Index)+=TmpColor1;
				}
				
				TmpColor1=(( (*(ptColor2+LineBytes-3+Index)- *(ptColor2+LineBytes-6+Index))+
					(*(ptColor2+LineBytes-3+Index)- *(ptColor1+LineBytes-3+Index))+
					(*(ptColor2+LineBytes-3+Index)- *(ptColor3+LineBytes-3+Index))    )) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*(ImgAddr+(Row+1)*LineBytes-3+Index));
				if( (TmpColor2+TmpColor1)<=0)
				{
					*(ImgAddr+(Row+1)*LineBytes-3+Index)=0;
				}
				else if((TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+(Row+1)*LineBytes-3+Index)=255;
				}
				else
				{
					*(ImgAddr+(Row+1)*LineBytes-3+Index)+=TmpColor1;
				}
			}

			for(Col=3;Col<(LineBytes-1);Col+=3)
			{				
				for(Index=0;Index<3;Index++)
				{
					TmpColor1=(( (*(ptColor2+Col+Index)- *(ptColor2+Col-3+Index))+
						(*(ptColor2+Col+Index)- *(ptColor2+Col+3+Index))+
						(*(ptColor2+Col+Index)- *(ptColor1+Col+Index))  +
						(*(ptColor2+Col+Index)- *(ptColor3+Col+Index)) )) ;
					TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
					
					TmpColor2=(BYTE)(*(ImgAddr+Row*LineBytes+Col+Index));
					if( (TmpColor2+TmpColor1)<=0)
					{
						*(ImgAddr+Row*LineBytes+Col+Index)=0;
					}
					else if((TmpColor2+TmpColor1) >=255 )
					{
						*(ImgAddr+Row*LineBytes+Col+Index)=255;
					}
					else
					{
						*(ImgAddr+Row*LineBytes+Col+Index)+=TmpColor1;
					}
				}
				
			}
		}

		for(Index=0;Index<3;Index++)
		{
			TmpColor1=(( (*(ptColor3+Index)- *(ptColor3+3+Index))+
				(*(ptColor3+Index)- *(ptColor2+Index  )) )) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes+Index));
			if( (TmpColor2+TmpColor1)<=0)
			{
				*(ImgAddr+(Lines-1)*LineBytes+Index)=0;
			}
			else if((TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Lines-1)*LineBytes+Index)=255;
			}
			else
			{
				*(ImgAddr+(Lines-1)*LineBytes+Index)+=TmpColor1;
			}
			
			TmpColor1=( ((*(ptColor3+LineBytes-3+Index) - *(ptColor3+LineBytes-6+Index))+
				(*(ptColor3+LineBytes-3+Index) - *(ptColor3+LineBytes-3+Index)))) ;
			TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
			
			TmpColor2=(BYTE)(*( ImgAddr+(Lines*LineBytes)-3+Index));
			if( ( TmpColor2+TmpColor1 )<=0  )
			{
				*(ImgAddr+(Lines*LineBytes)-3+Index)=0;
			}
			else if( (TmpColor2+TmpColor1) >=255 )
			{
				*(ImgAddr+(Lines*LineBytes)-3+Index)=255;
			}
			else
			{
				*(ImgAddr+(Lines*LineBytes)-3+Index)+=TmpColor1;
			}
			
			for(Col=3;Col< (LineBytes-1) ;Col+=3)
			{
				TmpColor1=( (  (*(ptColor3+Col+Index)-*(ptColor3+Col-3+Index))+
					(*(ptColor3+Col+Index)-*(ptColor3+Col+3+Index))+			        
					(*(ptColor3+Col+Index)-*(ptColor2+Col+Index)) ) ) ;
				TmpColor1*=(short)NormalInfo[abs(TmpColor1)];
				
				TmpColor2=(BYTE)(*(ImgAddr+(Lines-1)*LineBytes+Col+Index));
				if( ( TmpColor2+TmpColor1 )<=0  )
				{
					*(ImgAddr+(Lines-1)*LineBytes+Col+Index)=0;
				}
				else if( (TmpColor2+TmpColor1) >=255 )
				{
					*(ImgAddr+(Lines-1)*LineBytes+Col+Index)=255;
				}
				else
				{
					*(ImgAddr+(Lines-1)*LineBytes+Col+Index)+=TmpColor1;
				}
			}	
		}

		free(ptColor);
	}

}

void* MustScanner_ReadDataFromScanner()
{	
	WORD wTotalReadImageLines=0;
	WORD wWantedLines=g_Height;
	LPBYTE lpReadImage=g_lpReadImageHead;
	BOOL isWaitImageLineDiff=FALSE;
	WORD wMaxScanLines=g_wMaxScanLines;
	WORD wReadImageLines=0;
	WORD wScanLinesThisBlock;
	WORD wBufferLines=g_wLineDistance*2+g_wPixelDistance;

	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: call in, and in new thread");
	
	while(wTotalReadImageLines<wWantedLines&&g_lpReadImageHead)
	{			
		if(!isWaitImageLineDiff)
		{
			wScanLinesThisBlock=(wWantedLines-wTotalReadImageLines) < g_wScanLinesPerBlock ?
				(wWantedLines-wTotalReadImageLines) : g_wScanLinesPerBlock ;
		
			XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: wWantedLines=%d",wWantedLines);
		
			XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: wScanLinesThisBlock=%d",wScanLinesThisBlock);
	
			if(STATUS_GOOD!=Asic_ReadImage(&g_chip,lpReadImage,wScanLinesThisBlock))
			{
				XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner:Asic_ReadImage return error");
				XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner:thread exit");
				return NULL;
			}
			
			wReadImageLines+=wScanLinesThisBlock;   
			
			g_dwScannedTotalLines+=wScanLinesThisBlock;  
			
			wTotalReadImageLines+=wScanLinesThisBlock; 
			
			lpReadImage+=wScanLinesThisBlock*g_BytesPerRow;
			
			if(wReadImageLines>=wMaxScanLines) 
			{
				lpReadImage=g_lpReadImageHead;
				wReadImageLines=0;
			}	
						
			if((WORD)(g_dwScannedTotalLines - g_wtheReadyLines) 
				>= (wMaxScanLines - wBufferLines + g_wScanLinesPerBlock)
				&& g_dwScannedTotalLines > (DWORD)(g_wtheReadyLines) )
			{
				isWaitImageLineDiff=TRUE;  
			}
			
		}
		else if(g_dwScannedTotalLines <= (DWORD)(g_wtheReadyLines+wBufferLines+g_wScanLinesPerBlock))
		{
			isWaitImageLineDiff=FALSE;   
		}

		pthread_testcancel();  
	}

	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: Read image ok");
	XDBG(DEBUG_SPICALL,"MustScanner_ReadDataFromScanner: thread exit");

	return NULL;
}



BYTE  QBET4(BYTE A, BYTE B)
{
	BYTE bQBET[16][16]= {
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	0, 0, 0, 0, 1, 1, 2, 2, 4, 4, 5, 5, 8, 8, 9, 9,
	1, 1, 1, 1, 3, 3, 3, 3, 6, 6, 6, 6,10,10,11,11,
	1, 1, 1, 1, 3, 3, 3, 3, 6, 6, 6, 6,10,10,11,11,
	2, 2, 2, 2, 3, 3, 3, 3, 7, 7, 7, 7,10,10,11,11,
	2, 2, 2, 2, 3, 3, 3, 3, 7, 7, 7, 7,10,10,11,11,
	4, 4, 4, 4, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	4, 4, 4, 4, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	5, 5, 5, 5, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	5, 5, 5, 5, 6, 6, 7, 7,12,12,12,12,13,13,14,14,
	8, 8, 8, 8,10,10,10,10,13,13,13,13,15,15,15,15,
	8, 8, 8, 8,10,10,10,10,13,13,13,13,15,15,15,15,
	9, 9, 9, 9,11,11,11,11,14,14,14,14,15,15,15,15,
	9, 9, 9, 9,11,11,11,11,14,14,14,14,15,15,15,15};

	A=A&0x0f;
	B=B&0x0f;
	return bQBET[A][B];
}
